#!/usr/bin/env python

"""Tests for `ip_liberator` package."""
import typing
import unittest.mock

from unittest.mock import call

import botocore.exceptions
import pytest

from ip_liberator.ip_liberator import AwsIpLiberator

credentials = {'access_key': 'Gnawer Comade',
               'secret_key': '123456',
               'region_name': 'brazil-northeast'}


class TestAwsIpLiberator:
    def setup_method(self):
        with unittest.mock.patch('boto3.session.Session'):
            self.liberator = AwsIpLiberator(**credentials)

    def test_authorize_rule(self):
        # given
        mock_ec2 = self.liberator.ec2
        any_dict = {'a': 0, 'b': 1, 'c': 2}

        # when
        self.liberator.authorize_rule(any_dict)

        # then
        mock_ec2.authorize_security_group_ingress.assert_called_once_with(**any_dict)

    def test_authorize_rule__error_reraised(self):
        # given
        error = botocore.exceptions.ClientError(unittest.mock.NonCallableMagicMock(), 'test')
        mock_ec2 = self.liberator.ec2
        mock_ec2.authorize_security_group_ingress.side_effect = error

        # given
        any_dict = {'a': 0, 'b': 1, 'c': 2}

        # when..then
        with pytest.raises(botocore.exceptions.ClientError):
            self.liberator.authorize_rule(any_dict)

        # then
        mock_ec2.authorize_security_group_ingress.assert_called_once_with(**any_dict)

    @unittest.mock.patch('ip_liberator.ip_liberator.print')
    def test_authorize_rule__already_exists(self, mock_print):
        # given
        error = botocore.exceptions.ClientError({'Error': {'Code': 'InvalidPermission.Duplicate'}}, 'test')
        mock_ec2 = self.liberator.ec2
        mock_ec2.authorize_security_group_ingress.side_effect = error

        # given
        rule = {'IpPermissions': [{'IpRanges': [{'Description': 'Python is Cool', 'CidrIp': '1.2.3.4/8'}]}]}

        # when
        self.liberator.authorize_rule(rule)

        # then
        mock_ec2.authorize_security_group_ingress.assert_called_once_with(**rule)

        # then
        mock_print.assert_called_once_with(
            "> Rule 'Python is Cool' not set: permission with IP 1.2.3.4/8 already exists")

    def test_authorize_rule__authorized_individually(self):
        # given
        error = botocore.exceptions.ClientError({'Error': {'Code': 'InvalidPermission.Duplicate'}}, 'test')
        mock_ec2 = self.liberator.ec2
        mock_ec2.authorize_security_group_ingress.side_effect = (error, None, None)

        # given
        full_rule = {'GroupId': 'sg-1',
                     'IpPermissions': [{'IpRanges': [{'Description': 'Py3', 'CidrIp': '1.1.1.1/8'}]},
                                       {'IpRanges': [{'Description': '@->', 'CidrIp': '2.2.2.2/8'}]}]}
        rule1, rule2 = [{'GroupId': 'sg-1',
                         'IpPermissions': [p]} for p in full_rule['IpPermissions']]

        # when
        self.liberator.authorize_rule(full_rule)

        # then
        mock_ec2.authorize_security_group_ingress.assert_has_calls([call(**full_rule), call(**rule1), call(**rule2)])

    def test_revoke_rule(self):
        # given
        mock_ec2 = self.liberator.ec2

        # given
        any_dict = {'a': 0, 'b': 1, 'c': 2}

        # when
        self.liberator.revoke_rule(any_dict)

        # then
        mock_ec2.revoke_security_group_ingress.assert_called_once_with(**any_dict)

    def test_describe_rules(self):
        # given
        group_ids = ['sg-1', 'sg-2']
        services = ('SSH', 'HTTP')
        wanted = [{'Description': svc} for svc in services]

        # given
        def describe_rules_return(group_id: str, data: list) -> dict:
            return {'GroupId': group_id, 'IpPermissions': [{'IpRanges': data}]}

        # given
        unwanted = [{'Description': svc + '1'} for svc in services]
        aws_return = {'SecurityGroups': [describe_rules_return(group_ids[0], wanted),     # first is kept
                                         describe_rules_return(group_ids[1], unwanted)]}  # last is removed (unwanted)

        # given
        mock_ec2 = self.liberator.ec2
        mock_ec2.describe_security_groups.return_value = aws_return

        # when
        rules = self.liberator.describe_rules(services, {'security_groups': group_ids})

        # then
        assert isinstance(rules, typing.Iterator)

        # then
        assert next(rules) == describe_rules_return(group_ids[0], wanted)

        # then
        with pytest.raises(StopIteration):
            next(rules)

    @unittest.mock.patch('boto3.session.Session')
    def test_init(self, mock):
        # given
        mock_session = unittest.mock.NonCallableMock()
        mock.return_value = mock_session

        # when
        liberator = AwsIpLiberator(**credentials)

        # then
        assert liberator.ec2 == mock_session.client.return_value

        # then
        mock.assert_called_once_with(
            aws_access_key_id=credentials['access_key'],
            aws_secret_access_key=credentials['secret_key'],
            region_name=credentials['region_name']
        )
        mock_session.client.assert_called_once_with('ec2')



#!/usr/bin/env python

"""Tests for `ip_liberator.utils` package."""

import typing
import unittest.mock

import pytest

from ip_liberator.utils import whats_my_ip, make_keys, duplicate_removal, make_rule, make_rules


class TestWhatsMyIp:
    def setup_method(self):
        if hasattr(whats_my_ip, 'cache'):
            del whats_my_ip.cache

    @unittest.mock.patch('urllib.request.urlopen')
    def test_whats_my_ip(self, mock_urlopen):
        mock_response = unittest.mock.NonCallableMagicMock(status=200, **{'read.return_value': b'127.0.0.1'})
        mock_response.__enter__ = lambda self: self
        mock_urlopen.return_value = mock_response

        assert whats_my_ip() == '127.0.0.1/32'
        assert hasattr(whats_my_ip, 'cache')
        assert whats_my_ip.cache == '127.0.0.1/32'

        mock_urlopen.assert_called_once_with("https://checkip.amazonaws.com/")

    @unittest.mock.patch('urllib.request.urlopen')
    def test_whats_my_ip__fail(self, mock_urlopen):
        assert whats_my_ip() is None
        assert not hasattr(whats_my_ip, 'cache')

        mock_urlopen.assert_called_once_with("https://checkip.amazonaws.com/")

    def test_whats_my_ip__cache(self):
        value = 'Python is Cool'

        with unittest.mock.patch('urllib.request.urlopen'):
            assert whats_my_ip(value) == value
            assert hasattr(whats_my_ip, 'cache')
            assert whats_my_ip.cache == value


def test_make_keys():
    rule = {
        'IpPermissions': [make_rule('Python', '10.0.0.1/32', port=1),
                          make_rule('IsCool', '10.0.0.2/32', from_port=2, to_port=9)]
    }
    expected = {
        (1, 1, 'Python', '10.0.0.1/32'): 0,
        (2, 9, 'IsCool', '10.0.0.2/32'): 1,
    }

    assert make_keys(rule) == expected


def test_duplicate_removal():
    rule1 = {
        'IpPermissions': [make_rule('Python', '10.0.0.1/32', port=1),
                          make_rule('IsCool', '10.0.0.2/32', port=2)]
    }
    rule2 = {
        'IpPermissions': [make_rule('Python', '10.0.0.1/32', port=1),
                          make_rule('Second', '10.0.0.2/32', port=2),
                          make_rule('ToNone', '10.0.0.3/32', port=3)]
    }

    duplicate_removal(rule1, rule2)
    assert len(rule1['IpPermissions']) == 1
    assert len(rule2['IpPermissions']) == 2
    assert rule1['IpPermissions'] == [make_rule('IsCool', '10.0.0.2/32', port=2)]
    assert rule2['IpPermissions'] == [make_rule('Second', '10.0.0.2/32', port=2),
                                      make_rule('ToNone', '10.0.0.3/32', port=3)]


def test_make_rule():
    rule_http = make_rule('HTTP', '10.0.0.1/32', port=80)
    rule_many = make_rule('Many Services', '10.0.0.1/32', from_port=1, to_port=1000)

    assert rule_http == {'FromPort': 80, 'ToPort': 80, 'IpProtocol': 'tcp',
                         'IpRanges': [{'CidrIp': '10.0.0.1/32', 'Description': 'HTTP'}]}
    assert rule_many == {'FromPort': 1, 'ToPort': 1000, 'IpProtocol': 'tcp',
                         'IpRanges': [{'CidrIp': '10.0.0.1/32', 'Description': 'Many Services'}]}


@unittest.mock.patch('ip_liberator.utils.whats_my_ip')
def test_make_rules(mock_ip):
    mock_ip.return_value = '10.0.0.1/32'

    rules = make_rules(config={"security_groups": ["sg-1"]},
                       services={"John SFTP": {"name": "SFTP", "port": "22"},
                                 "John HTTP": {"name": "HTTP", "port": "80"}})
    expected = {'GroupId': 'sg-1',
                'IpPermissions': [
                    {'FromPort': 22, 'ToPort': 22, 'IpProtocol': 'tcp',
                     'IpRanges': [{'CidrIp': '10.0.0.1/32', 'Description': 'John SFTP'}]},
                    {'FromPort': 80, 'ToPort': 80, 'IpProtocol': 'tcp',
                     'IpRanges': [{'CidrIp': '10.0.0.1/32', 'Description': 'John HTTP'}]}
                ]}

    assert isinstance(rules, typing.Iterator)
    assert next(rules) == expected
    with pytest.raises(StopIteration): next(rules)


@unittest.mock.patch('ip_liberator.utils.whats_my_ip')
def test_make_rules__no_port_range(mock_ip):
    mock_ip.return_value = '10.0.0.1/32'

    ports_input = [
        "",
        " ",
        "\t\n "
    ]

    for port in ports_input:
        rules = make_rules(config={"security_groups": ["sg-1"]},
                           services={"SVC": {"port": port}})

        with pytest.raises(ValueError, match="No port range informed in service: SVC"):
            next(rules)



@unittest.mock.patch('ip_liberator.utils.whats_my_ip')
def test_make_rules__invalid_port_range(mock_ip):
    mock_ip.return_value = '10.0.0.1/32'

    ports_input = [
        "1-2-3",
        "100-",
        "-100",
        "abc",
        "1 2 3",
    ]

    for port in ports_input:
        rules = make_rules(config={"security_groups": ["sg-1"]},
                           services={"SVC": {"port": port}})

        with pytest.raises(ValueError, match="Invalid port range: '%s'" % port):
            next(rules)



#!/usr/bin/env python

"""Tests for `ip_liberator.__main__` package."""
import json
import os
import tempfile
from unittest import mock

from ip_liberator.__main__ import main, make_services_index
from ip_liberator.utils import make_rules, make_rule

IP = '127.0.0.1/8'


@mock.patch('ip_liberator.__main__.AwsIpLiberator')
@mock.patch('ip_liberator.__main__.print')
@mock.patch('ip_liberator.utils.whats_my_ip', lambda: IP)
class TestMain:
    def setup_method(self):
        (self.fd, self.filename) = tempfile.mkstemp()

    def teardown_method(self):
        os.remove(self.filename)

    def test_main(self, mock_print, mock_aws_class):
        # given
        access_key = "LT7F9TDQ"
        secret_key = "SAERB0DITUERDQTYYX8Q"
        region_name = "sa-west-1"
        operator = "Fighter"
        services = [{"name": "HTTP", "port": "80"}]
        security_groups = ["sg-1"]

        # given
        settings = make_settings(access_key, secret_key, region_name, operator, services, security_groups)
        index = make_services_index(settings)
        rule = next(make_rules(index, settings['config']))

        # given
        with os.fdopen(self.fd, mode='w') as file:
            json.dump(settings, file)

        # when
        main(args=["--profile", self.filename])

        # then
        mock_print.assert_has_calls([mock.call("Authorizing rules", [svc for svc in index], "to IP", IP),
                                     mock.call('-', security_groups[0])])

        # then
        mock_aws_class.assert_called_once_with(access_key, secret_key, region_name)
        mock_liberator = mock_aws_class.return_value
        mock_liberator.describe_rules.assert_called_once_with(index, settings['config'])
        mock_liberator.authorize_rule.assert_called_once_with(rule)

    def test_main__revoke_only(self, mock_print, mock_aws_class):
        # given
        group_id = "sg-1"
        rule_to_revoke = {'GroupId': group_id}
        settings = make_settings(security_groups=[group_id])
        index = make_services_index(settings)

        # given
        with os.fdopen(self.fd, mode='w') as file:
            json.dump(settings, file)

        # given
        mock_liberator = mock_aws_class.return_value
        mock_liberator.describe_rules.return_value = [rule_to_revoke]

        # when
        main(args=["--profile", self.filename, "--revoke-only"])

        # then
        mock_print.assert_has_calls([mock.call("Revoking rules", [svc for svc in index]),
                                     mock.call('-', group_id)])

        # then
        mock_liberator.describe_rules.assert_called_once_with(index, settings['config'])
        mock_liberator.revoke_rule.assert_called_once_with(rule_to_revoke)

    def test_main__duplicate_rule(self, mock_print, mock_aws_class):
        # given
        group_id = "sg-1"
        rule_to_revoke = {'GroupId': group_id,
                          'IpPermissions': [make_rule('Monty Python', IP, port=1),
                                            make_rule('Monty Second', IP, port=2),
                                            make_rule('Monty ToNone', IP, port=3)]}
        settings = make_settings(security_groups=[group_id],
                                 operator='Monty',
                                 services=[{"name": "Python", "port": "1"},
                                           {"name": "IsCool", "port": "4"}])
        index = make_services_index(settings)

        # given
        with os.fdopen(self.fd, mode='w') as file:
            json.dump(settings, file)

        # given
        mock_liberator = mock_aws_class.return_value
        mock_liberator.describe_rules.return_value = [rule_to_revoke]

        # when
        main(args=["--profile", self.filename])

        # then
        mock_print.assert_has_calls([mock.call("Authorizing rules", [svc for svc in index], "to IP", IP),
                                     mock.call('-', group_id)])

        # then
        mock_liberator.describe_rules.assert_called_once_with(index, settings['config'])
        mock_liberator.authorize_rule.assert_called_once_with({'GroupId': group_id,
                                                               'IpPermissions': [make_rule('Monty IsCool', IP, port=4)]})
        mock_liberator.revoke_rule.assert_called_once_with(rule_to_revoke)

    @mock.patch('ip_liberator.__main__.whats_my_ip')
    def test_main__my_ip(self, mock_whats_my_ip, *_):
        # given
        with os.fdopen(self.fd, mode='w') as file:
            json.dump(make_settings(), file)

        # given
        my_ip = "1.2.3.4/32"

        # when
        main(args=["--profile", self.filename, "--my-ip", my_ip])

        # then
        mock_whats_my_ip.assert_called_once_with(my_ip)

    def test_main__ip_informed_at_service(self, mock_print, mock_aws_class):
        # given
        group_id = "sg-1"
        informed_ip = "1.2.3.4/32"
        operator = "Peter"
        services = [{"name": "HTTP", "port": "80"},
                    {"name": "SFTP", "port": "22", "ip": informed_ip}]

        # given
        descriptions = ["%s %s" % (operator, svc["name"]) for svc in services]
        settings = make_settings(operator=operator, services=services, security_groups=[group_id])
        services_index = make_services_index(settings)
        rule = next(make_rules(services_index, settings['config']))

        # given
        with os.fdopen(self.fd, mode='w') as file:
            json.dump(settings, file)

        # when
        main(args=["--profile", self.filename])

        # then
        mock_print.assert_has_calls([mock.call("Authorizing rule '%s' to IP %s" % (descriptions[0], IP)),
                                     mock.call("Authorizing rule '%s' to IP %s" % (descriptions[1], informed_ip)),
                                     mock.call('-', group_id)])

        # then
        mock_liberator = mock_aws_class.return_value
        mock_liberator.describe_rules.assert_called_once_with(services_index, settings['config'])
        mock_liberator.authorize_rule.assert_called_once_with(rule)


def make_settings(access_key: str = None, secret_key: str = None, region_name: str = None,
                  operator: str = None, services: list = None, security_groups: list = None):
    return {
        "credentials": {
            "access_key": access_key or '',
            "secret_key": secret_key or '',
            "region_name": region_name or '',
        },
        "config": {
            "operator": operator or '',
            "services": services or [{"name": "example", "port": "255"}],
            "security_groups": security_groups or ["sg-x"],
        }
    }



#!/usr/bin/env python3

"""The setup script."""

from setuptools import setup, find_packages

version = '0.2.0'

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as requirements_file:
    requirements = requirements_file.read().splitlines()

setup_requirements = ['pytest-runner', ]
test_requirements = ['pytest', ]

setup(
    name="ip-liberator",
    version=version,
    author="Wagner Macedo",
    author_email="wagnerluis1982@gmail.com",
    description="A command line utility to update AWS Security Groups rules.",
    long_description=readme + '\n\n' + history,
    py_modules = ['ip_liberator'],
    entry_points={
        'console_scripts': [
            'ip-liberator = ip_liberator.__main__:main'
        ],
    },
    install_requires=requirements,
    setup_requires=setup_requirements,
    include_package_data=True,
    packages=find_packages(include=['ip_liberator']),
    test_suite='tests',
    tests_require=test_requirements,
    license="GNU General Public License v3",
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Topic :: Utilities",
    ],
    url='https://github.com/wagnerluis1982/ip-liberator',
    keywords='ip-liberator, Python, Amazon AWS, cloud computing, DevOps',
)
